@@ -436,6 +436,15 @@ impl Client {
                     let _ = self.sender.try_send(command!("MODE", nick, modestring));
                 }
 
+                // Loop on connect commands
+                for command in self.config.on_connect.iter() {
+                    if let Ok(cmd) = crate::command::parse(command, None) {
+                        if let Ok(command) = proto::Command::try_from(cmd) {
+                            let _ = self.sender.try_send(command.into());
+                        };
+                    };
+                }
+
                 // Send JOIN
                 for channel in &self.config.channels {
                     if let Some(keys) = self.config.channel_keys.get(channel) {
@@ -506,6 +515,11 @@ impl Client {
                             user.update_away(away);
                             channel.users.insert(user);
                         }
+
+                        // We requested, don't save to history
+                        if matches!(channel.last_who, Some(WhoStatus::Receiving)) {
+                            return None;
+                        }
                     }
                 }
             }
@@ -514,8 +528,11 @@ impl Client {
 
                 if proto::is_channel(target) {
                     if let Some(channel) = self.chanmap.get_mut(target) {
-                        channel.last_who = Some(WhoStatus::Done(Instant::now()));
-                        log::debug!("[{}] {target} - WHO done", self.server);
+                        if matches!(channel.last_who, Some(WhoStatus::Receiving)) {
+                            channel.last_who = Some(WhoStatus::Done(Instant::now()));
+                            log::debug!("[{}] {target} - WHO done", self.server);
+                            return None;
+                        }
                     }
                 }
             }
@@ -582,6 +599,9 @@ impl Client {
                     }
                 }
             }
+            #[cfg(feature = "dev")]
+            // Suppress topic during development to prevent history spam
+            Command::Numeric(RPL_TOPIC | RPL_TOPICWHOTIME, _) => return None,
             _ => {}
         }
 
@@ -746,6 +766,16 @@ impl Map {
             .unwrap_or_default()
     }
 
+    pub fn connected_servers(&self) -> impl Iterator<Item = &Server> {
+        self.0.iter().filter_map(|(server, state)| {
+            if let State::Ready(_) = state {
+                Some(server)
+            } else {
+                None
+            }
+        })
+    }
+
     pub fn iter(&self) -> std::collections::btree_map::Iter<Server, State> {
         self.0.iter()
     }
