@@ -1,3 +1,4 @@
+mod command_bar;
 pub mod pane;
 pub mod side_menu;
 
@@ -5,15 +6,17 @@ use std::time::{Duration, Instant};
 
 use data::history::manager::Broadcast;
 use data::user::Nick;
-use data::{history, Config, Server, User};
+use data::{client, history, server, Config, Server, User};
 use iced::widget::pane_grid::{self, PaneGrid};
-use iced::widget::{container, row};
+use iced::widget::{container, row, Space};
 use iced::{clipboard, window, Command, Length};
-use pane::Pane;
-use side_menu::SideMenu;
 
+use self::command_bar::CommandBar;
+use self::pane::Pane;
+use self::side_menu::SideMenu;
 use crate::buffer::{self, Buffer};
-use crate::widget::{selectable_text, Element};
+use crate::widget::{anchored_overlay, selectable_text, shortcut, Collection, Element};
+use crate::{event, theme, Theme};
 
 const SAVE_AFTER: Duration = Duration::from_secs(3);
 
@@ -23,6 +26,7 @@ pub struct Dashboard {
     side_menu: SideMenu,
     history: history::Manager,
     last_changed: Option<Instant>,
+    command_bar: Option<CommandBar>,
 }
 
 #[derive(Debug)]
@@ -35,6 +39,8 @@ pub enum Message {
     DashboardSaved(Result<(), data::dashboard::Error>),
     CloseHistory,
     QuitServer,
+    Command(command_bar::Message),
+    Shortcut(shortcut::Command),
 }
 
 impl Dashboard {
@@ -47,6 +53,7 @@ impl Dashboard {
             side_menu: SideMenu::new(),
             history: history::Manager::default(),
             last_changed: None,
+            command_bar: None,
         };
 
         let command = dashboard.track();
@@ -65,8 +72,9 @@ impl Dashboard {
     pub fn update(
         &mut self,
         message: Message,
-        clients: &mut data::client::Map,
-        servers: &mut data::server::Map,
+        clients: &mut client::Map,
+        servers: &mut server::Map,
+        theme: &mut Theme,
         config: &Config,
     ) -> Command<Message> {
         match message {
@@ -85,25 +93,11 @@ impl Dashboard {
                 pane::Message::PaneDragged(_) => {}
                 pane::Message::ClosePane => {
                     if let Some(pane) = self.focus {
-                        self.last_changed = Some(Instant::now());
-
-                        if let Some((_, sibling)) = self.panes.close(&pane) {
-                            return self.focus_pane(sibling);
-                        } else if let Some(pane) = self.panes.get_mut(&pane) {
-                            pane.buffer = Buffer::Empty;
-                        }
+                        return self.close_pane(pane);
                     }
                 }
                 pane::Message::SplitPane(axis) => {
-                    if let Some(pane) = self.focus {
-                        let result =
-                            self.panes
-                                .split(axis, &pane, Pane::new(Buffer::Empty, config));
-                        self.last_changed = Some(Instant::now());
-                        if let Some((pane, _)) = result {
-                            return self.focus_pane(pane);
-                        }
-                    }
+                    return self.split_pane(axis, config);
                 }
                 pane::Message::Buffer(id, message) => {
                     if let Some(pane) = self.panes.get_mut(&id) {
@@ -140,6 +134,17 @@ impl Dashboard {
                                         return self.open_buffer(buffer, config);
                                     }
                                 }
+                                buffer::user_context::Event::SingleClick(user) => {
+                                    let Some((_, pane)) = self.get_focused_mut() else {
+                                        return Command::none();
+                                    };
+
+                                    return pane.buffer.insert_user_to_input(user).map(
+                                        move |message| {
+                                            Message::Pane(pane::Message::Buffer(id, message))
+                                        },
+                                    );
+                                }
                             }
                         }
 
@@ -153,13 +158,7 @@ impl Dashboard {
                         self.last_changed = Some(Instant::now());
                     }
                 }
-                pane::Message::MaximizePane => {
-                    if self.panes.maximized().is_some() {
-                        self.panes.restore();
-                    } else if let Some(pane) = self.focus {
-                        self.panes.maximize(&pane);
-                    }
-                }
+                pane::Message::MaximizePane => self.maximize_pane(),
             },
             Message::SideMenu(message) => {
                 let event = self.side_menu.update(message);
@@ -301,6 +300,160 @@ impl Dashboard {
             }
             Message::CloseHistory => {}
             Message::QuitServer => {}
+            Message::Command(message) => {
+                let Some(command_bar) = &mut self.command_bar else {
+                    return Command::none();
+                };
+
+                match command_bar.update(message) {
+                    Some(command_bar::Event::ThemePreview(preview)) => match preview {
+                        Some(preview) => *theme = theme.preview(preview),
+                        None => *theme = theme.selected(),
+                    },
+                    Some(command_bar::Event::Command(command)) => {
+                        let command = match command {
+                            command_bar::Command::Buffer(command) => match command {
+                                command_bar::Buffer::Maximize(_) => {
+                                    self.maximize_pane();
+                                    Command::none()
+                                }
+                                command_bar::Buffer::New => {
+                                    self.new_pane(pane_grid::Axis::Horizontal, config)
+                                }
+                                command_bar::Buffer::Close => {
+                                    if let Some(pane) = self.focus {
+                                        self.close_pane(pane)
+                                    } else {
+                                        Command::none()
+                                    }
+                                }
+                                command_bar::Buffer::Replace(buffer) => {
+                                    let mut commands = vec![];
+
+                                    if let Some(pane) = self.focus.take() {
+                                        if let Some(state) = self.panes.get_mut(&pane) {
+                                            state.buffer = Buffer::from(buffer);
+                                            self.last_changed = Some(Instant::now());
+
+                                            commands.extend(vec![
+                                                self.reset_pane(pane),
+                                                self.focus_pane(pane),
+                                            ]);
+                                        }
+                                    }
+
+                                    Command::batch(commands)
+                                }
+                            },
+                            command_bar::Command::Configuration(command) => match command {
+                                command_bar::Configuration::Open => {
+                                    let _ = open::that(Config::config_dir());
+                                    Command::none()
+                                }
+                            },
+                            command_bar::Command::UI(command) => match command {
+                                command_bar::Ui::ToggleSidebarVisibility => {
+                                    self.side_menu.toggle_visibility();
+                                    Command::none()
+                                }
+                            },
+                            command_bar::Command::Theme(command) => match command {
+                                command_bar::Theme::Switch(new) => {
+                                    *theme = Theme::from(new);
+                                    Command::none()
+                                }
+                            },
+                        };
+
+                        return Command::batch(vec![
+                            command,
+                            self.toggle_command_bar(&closed_buffers(self, clients), config, theme),
+                        ]);
+                    }
+                    Some(command_bar::Event::Unfocused) => {
+                        return self.toggle_command_bar(
+                            &closed_buffers(self, clients),
+                            config,
+                            theme,
+                        );
+                    }
+                    None => {}
+                }
+            }
+            Message::Shortcut(shortcut) => {
+                use shortcut::Command::*;
+
+                let mut move_focus = |direction: pane_grid::Direction| {
+                    if let Some(pane) = self.focus.as_ref() {
+                        if let Some(adjacent) = self.panes.adjacent(pane, direction) {
+                            return self.focus_pane(adjacent);
+                        }
+                    } else if let Some((pane, _)) = self.panes.panes.iter().next() {
+                        return self.focus_pane(*pane);
+                    }
+
+                    Command::none()
+                };
+
+                match shortcut {
+                    MoveUp => return move_focus(pane_grid::Direction::Up),
+                    MoveDown => return move_focus(pane_grid::Direction::Down),
+                    MoveLeft => return move_focus(pane_grid::Direction::Left),
+                    MoveRight => return move_focus(pane_grid::Direction::Right),
+                    CloseBuffer => {
+                        if let Some(pane) = self.focus {
+                            return self.close_pane(pane);
+                        }
+                    }
+                    MaximizeBuffer => {
+                        if let Some(pane) = self.focus.as_ref() {
+                            self.panes.maximize(pane);
+                        }
+                    }
+                    RestoreBuffer => {
+                        self.panes.restore();
+                    }
+                    CycleNextBuffer => {
+                        let all_buffers = all_buffers(clients, &self.history);
+                        let open_buffers = open_buffers(self);
+
+                        if let Some((pane, state)) = self.get_focused_mut() {
+                            if let Some(buffer) = cycle_next_buffer(
+                                state.buffer.data().as_ref(),
+                                all_buffers,
+                                &open_buffers,
+                            ) {
+                                state.buffer = Buffer::from(buffer);
+                                self.focus = None;
+                                return self.focus_pane(pane);
+                            }
+                        }
+                    }
+                    CyclePreviousBuffer => {
+                        let all_buffers = all_buffers(clients, &self.history);
+                        let open_buffers = open_buffers(self);
+
+                        if let Some((pane, state)) = self.get_focused_mut() {
+                            if let Some(buffer) = cycle_previous_buffer(
+                                state.buffer.data().as_ref(),
+                                all_buffers,
+                                &open_buffers,
+                            ) {
+                                state.buffer = Buffer::from(buffer);
+                                self.focus = None;
+                                return self.focus_pane(pane);
+                            }
+                        }
+                    }
+                    ToggleNickList => {
+                        if let Some((_, pane)) = self.get_focused_mut() {
+                            pane.update_settings(|settings| {
+                                settings.channel.users.visible = !settings.channel.users.visible
+                            });
+                        }
+                    }
+                }
+            }
         }
 
         Command::none()
@@ -308,7 +461,7 @@ impl Dashboard {
 
     pub fn view<'a>(
         &'a self,
-        clients: &'a data::client::Map,
+        clients: &'a client::Map,
         config: &'a Config,
     ) -> Element<'a, Message> {
         let focus = self.focus;
@@ -346,30 +499,71 @@ impl Dashboard {
                 self.focus,
                 config.dashboard.sidebar_default_action,
             )
-            .map(Message::SideMenu);
+            .map(|e| e.map(Message::SideMenu));
 
         // The height margin varies across different operating systems due to design differences.
         // For instance, on macOS, the menubar is hidden, resulting in a need for additional padding to accommodate the
         // space occupied by the traffic light buttons.
         let height_margin = if cfg!(target_os = "macos") { 20 } else { 0 };
 
-        row![side_menu, pane_grid]
+        let base = row![]
+            .push_maybe(side_menu)
+            .push(pane_grid)
             .width(Length::Fill)
             .height(Length::Fill)
-            .padding([height_margin, 0, 0, 0])
-            .into()
+            .padding([height_margin, 0, 0, 0]);
+
+        let base = if let Some(command_bar) = self.command_bar.as_ref() {
+            let background = anchored_overlay(
+                base,
+                container(Space::new(Length::Fill, Length::Fill))
+                    .width(Length::Fill)
+                    .height(Length::Fill)
+                    .style(theme::Container::SemiTransparent),
+                anchored_overlay::Anchor::BelowTopCentered,
+                0.0,
+            );
+
+            // Command bar
+            anchored_overlay(
+                background,
+                command_bar
+                    .view(
+                        &all_buffers(clients, &self.history),
+                        self.focus.is_some(),
+                        self.buffer_resize_action(),
+                        config,
+                    )
+                    .map(Message::Command),
+                anchored_overlay::Anchor::BelowTopCentered,
+                10.0,
+            )
+        } else {
+            base.into()
+        };
+
+        shortcut(base, config.keys.shortcuts(), Message::Shortcut)
     }
 
-    pub fn handle_event(&mut self, event: crate::event::Event) -> Command<Message> {
-        use crate::event::Event::*;
+    pub fn handle_event(
+        &mut self,
+        event: event::Event,
+        clients: &data::client::Map,
+        config: &Config,
+        theme: &mut Theme,
+    ) -> Command<Message> {
+        use event::Event::*;
 
         match event {
             Escape => {
                 // Order of operations
                 //
+                // - Close command bar
                 // - Restore maximized pane
                 // - Unfocus
-                if self.panes.maximized().is_some() {
+                if self.command_bar.is_some() {
+                    return self.toggle_command_bar(&closed_buffers(self, clients), config, theme);
+                } else if self.is_pane_maximized() {
                     self.panes.restore();
                 } else {
                     self.focus = None;
@@ -417,6 +611,7 @@ impl Dashboard {
 
                 Command::perform(task, |_| Message::Close)
             }
+            CommandBar => self.toggle_command_bar(&closed_buffers(self, clients), config, theme),
         }
     }
 
@@ -559,6 +754,60 @@ impl Dashboard {
         }
     }
 
+    fn maximize_pane(&mut self) {
+        if self.is_pane_maximized() {
+            self.panes.restore();
+        } else if let Some(pane) = self.focus {
+            self.panes.maximize(&pane);
+        }
+    }
+
+    fn is_pane_maximized(&self) -> bool {
+        self.panes.maximized().is_some()
+    }
+
+    fn new_pane(&mut self, axis: pane_grid::Axis, config: &Config) -> Command<Message> {
+        if self.focus.is_some() {
+            // If there is any focused pane, split it
+            return self.split_pane(axis, config);
+        } else {
+            // If there is no focused pane, split the last pane or create a new empty grid
+            let pane = self.panes.iter().last().map(|(pane, _)| pane).cloned();
+
+            if let Some(pane) = pane {
+                let result = self
+                    .panes
+                    .split(axis, &pane, Pane::new(Buffer::Empty, config));
+                self.last_changed = Some(Instant::now());
+
+                if let Some((pane, _)) = result {
+                    return self.focus_pane(pane);
+                }
+            } else {
+                let (state, pane) = pane_grid::State::new(Pane::new(Buffer::Empty, config));
+                self.panes = state;
+                self.last_changed = Some(Instant::now());
+                return self.focus_pane(pane);
+            }
+        }
+
+        Command::none()
+    }
+
+    fn split_pane(&mut self, axis: pane_grid::Axis, config: &Config) -> Command<Message> {
+        if let Some(pane) = self.focus {
+            let result = self
+                .panes
+                .split(axis, &pane, Pane::new(Buffer::Empty, config));
+            self.last_changed = Some(Instant::now());
+            if let Some((pane, _)) = result {
+                return self.focus_pane(pane);
+            }
+        }
+
+        Command::none()
+    }
+
     fn reset_pane(&mut self, pane: pane_grid::Pane) -> Command<Message> {
         self.panes
             .iter()
@@ -573,6 +822,18 @@ impl Dashboard {
             .unwrap_or(Command::none())
     }
 
+    fn close_pane(&mut self, pane: pane_grid::Pane) -> Command<Message> {
+        self.last_changed = Some(Instant::now());
+
+        if let Some((_, sibling)) = self.panes.close(&pane) {
+            return self.focus_pane(sibling);
+        } else if let Some(pane) = self.panes.get_mut(&pane) {
+            pane.buffer = Buffer::Empty;
+        }
+
+        Command::none()
+    }
+
     pub fn track(&mut self) -> Command<Message> {
         let resources = self
             .panes
@@ -613,6 +874,47 @@ impl Dashboard {
 
         history
     }
+
+    pub fn toggle_command_bar(
+        &mut self,
+        buffers: &[data::Buffer],
+        config: &Config,
+        theme: &mut Theme,
+    ) -> Command<Message> {
+        if self.command_bar.is_some() {
+            // Remove theme preview
+            *theme = theme.selected();
+
+            self.close_command_bar();
+            // Refocus the pane so text input gets refocused
+            self.focus
+                .take()
+                .map(|pane| self.focus_pane(pane))
+                .unwrap_or(Command::none())
+        } else {
+            self.open_command_bar(buffers, config);
+
+            Command::none()
+        }
+    }
+
+    fn open_command_bar(&mut self, buffers: &[data::Buffer], config: &Config) {
+        self.command_bar = Some(CommandBar::new(
+            buffers,
+            config,
+            self.focus.is_some(),
+            self.buffer_resize_action(),
+        ));
+    }
+
+    fn close_command_bar(&mut self) {
+        self.command_bar = None;
+    }
+
+    fn buffer_resize_action(&self) -> data::buffer::Resize {
+        let can_resize_buffer = self.focus.is_some() && self.panes.len() > 1;
+        data::buffer::Resize::action(can_resize_buffer, self.is_pane_maximized())
+    }
 }
 
 impl From<data::Dashboard> for Dashboard {
@@ -646,6 +948,7 @@ impl From<data::Dashboard> for Dashboard {
             side_menu: SideMenu::new(),
             history: history::Manager::default(),
             last_changed: None,
+            command_bar: None,
         }
     }
 }
@@ -682,3 +985,74 @@ impl<'a> From<&'a Dashboard> for data::Dashboard {
         }
     }
 }
+
+fn all_buffers(clients: &client::Map, history: &history::Manager) -> Vec<data::Buffer> {
+    clients
+        .connected_servers()
+        .flat_map(|server| {
+            std::iter::once(data::Buffer::Server(server.clone()))
+                .chain(
+                    clients
+                        .get_channels(server)
+                        .iter()
+                        .map(|channel| data::Buffer::Channel(server.clone(), channel.clone())),
+                )
+                .chain(
+                    history
+                        .get_unique_queries(server)
+                        .into_iter()
+                        .map(|nick| data::Buffer::Query(server.clone(), nick.clone())),
+                )
+        })
+        .collect()
+}
+
+fn open_buffers(dashboard: &Dashboard) -> Vec<data::Buffer> {
+    dashboard
+        .panes
+        .iter()
+        .filter_map(|(_, pane)| pane.buffer.data())
+        .collect()
+}
+
+fn closed_buffers(dashboard: &Dashboard, clients: &client::Map) -> Vec<data::Buffer> {
+    let open_buffers = open_buffers(dashboard);
+
+    all_buffers(clients, &dashboard.history)
+        .into_iter()
+        .filter(|buffer| !open_buffers.contains(buffer))
+        .collect()
+}
+
+fn cycle_next_buffer(
+    current: Option<&data::Buffer>,
+    mut all: Vec<data::Buffer>,
+    opened: &[data::Buffer],
+) -> Option<data::Buffer> {
+    all.retain(|buffer| Some(buffer) == current || !opened.contains(buffer));
+
+    let next = || {
+        let buffer = current?;
+        let index = all.iter().position(|b| b == buffer)?;
+        all.get(index + 1)
+    };
+
+    next().or_else(|| all.first()).cloned()
+}
+
+fn cycle_previous_buffer(
+    current: Option<&data::Buffer>,
+    mut all: Vec<data::Buffer>,
+    opened: &[data::Buffer],
+) -> Option<data::Buffer> {
+    all.retain(|buffer| Some(buffer) == current || !opened.contains(buffer));
+
+    let previous = || {
+        let buffer = current?;
+        let index = all.iter().position(|b| b == buffer).filter(|i| *i > 0)?;
+
+        all.get(index - 1)
+    };
+
+    previous().or_else(|| all.last()).cloned()
+}
